/*
 * Part of the GaBIEn project.
 * (Graphics and Basic Input Engine)
 *
 *        DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *                    Version 2, December 2004 
 *
 * Modified by gamemanj <asdd2808@gmail.com> to remove expletives
 * Original copyright (C) 2004 Sam Hocevar <sam@hocevar.net> 
 *
 * Everyone is permitted to copy and distribute verbatim or modified 
 * copies of this license document, and changing it is allowed as long 
 * as the name is changed. 
 *
 *            DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION 
 *
 *  0. You just DO WHATEVER YOU WANT TO.
 */
package gabien;

import gabien.IGrInDriver;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.lang.reflect.Field;
import java.util.Hashtable;
import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * 
 * @author matt
 */
final class GrInDriver implements IGrInDriver {
    public JFrame Frame;
    public JPanel Panel;
    public String typed="";
    public Graphics g;
    public BufferedImage bi;
    public Hashtable<String,IImage> ht=new Hashtable<String,IImage>();
    private boolean[] keys=new boolean[256];
    private boolean[] keysjd=new boolean[256];
    private int sc;
    private int RealWidth,RealHeight,MouseX=0,MouseY=0,MouseB=0;
    private boolean MouseDown=false,MouseJustDown=false;
    public GrInDriver(String name,int scale,int rw,int rh)
    {
        RealWidth=rw;
        RealHeight=rh;
        sc=scale;
        Frame=new JFrame(name);
        Panel=new JPanel();
        Frame.setResizable(false);
        
        Panel.setPreferredSize(new Dimension(rw*scale,rh*scale));
        Frame.setSize(rw*scale,rh*scale);
        
        Frame.add(Panel,BorderLayout.CENTER);
        Frame.pack();
        Frame.setLocationRelativeTo( null );
        Frame.setVisible(true);
        Panel.addMouseListener(new MouseListener() {

            @Override
            public void mouseClicked(MouseEvent me) {
            }

            @Override
            public void mousePressed(MouseEvent me) {
                MouseX=me.getX()/sc;
                MouseY=me.getY()/sc;
                MouseB=me.getButton();
                MouseDown=true;
                MouseJustDown=true;
            }

            @Override
            public void mouseReleased(MouseEvent me) {
                MouseX=me.getX()/sc;
                MouseY=me.getY()/sc;
                MouseDown=false;
                //justdown is a click checker.as in,even if the mouse is released,register the click
            }

            @Override
            public void mouseEntered(MouseEvent me) {
            }

            @Override
            public void mouseExited(MouseEvent me) {
            }
        });
        Panel.addMouseMotionListener(new MouseMotionListener(){

            @Override
            public void mouseDragged(MouseEvent me) {
                MouseX=me.getX()/sc;
                MouseY=me.getY()/sc;
                
            }

            @Override
            public void mouseMoved(MouseEvent me) {
                MouseX=me.getX()/sc;
                MouseY=me.getY()/sc;
            }
        });
        Frame.addKeyListener(new KeyListener(){

            @Override
            public void keyTyped(KeyEvent ke) {
                if ((ke.getKeyChar()>=32)||(ke.getKeyChar()==10))
                typed+=ke.getKeyChar();
            }
            //1:ESC 2:L 4:R 8:Items(s) 16:Swim!(z) 32:Mouse
            public int FilterKey(KeyEvent ke)
            {
                        try {
                String KeyID=null;
                for (Field f:KeyEvent.class.getFields())
                {
                   if (f.getName().startsWith("VK_"))
                   {
                            if (f.getInt(null)==ke.getKeyCode())
                            {
                                KeyID=f.getName();
                            }
                   }
                }
                if (KeyID==null) return -1;
                return IGrInDriver.class.getField(KeyID).getInt(null);
                        } catch (Exception ex) {
                        return -1;
                        }
            }
            @Override
            public void keyPressed(KeyEvent ke) {
                int KeyNum=FilterKey(ke);
                if (KeyNum!=-1)
                {
                if (!keys[KeyNum])
                    keysjd[KeyNum]=true;
                keys[KeyNum]=true;
                }
            }

            @Override
            public void keyReleased(KeyEvent ke) {
                int KeyNum=FilterKey(ke);
                if (KeyNum!=-1)
                keys[KeyNum]=false;
            }
        });
                
        bi=new BufferedImage(RealWidth,RealHeight,BufferedImage.TYPE_INT_RGB);
        g=bi.getGraphics();
    }

    @Override
    public void Flush() {
        Panel.getGraphics().drawImage(bi, 0,0,RealWidth*sc,RealHeight*sc,null);
    }

    @Override
    public void Clear(int i, int i0, int i1) {
        g.setColor(new Color(i,i0,i1));
        g.fillRect(0, 0, RealWidth, RealHeight);
    }

    @Override
    public int GetWidth() {
        return RealWidth;
    }

    @Override
    public int GetHeight() {
        return RealHeight;
    }

    @Override
    public void Display8pxText(int x, int y, int r, int cg, int b, String text) {
        try {
            g.setFont(Font.getFont(Font.MONOSPACED));
            g.setColor(new Color(r,cg,b));
            g.drawString(text, x, y+8);
        } catch (Exception ex) {
        }
    }

    @Override
    public int GetMouseX() {
        return MouseX;
    }

    @Override
    public int GetMouseY() {
        return MouseY;
    }

    @Override
    public int GetMouseButton() {
        return MouseB;
    }

    @Override
    public boolean StillRunning() {
        return Frame.isVisible();
    }
    
    @Override
    public boolean IsKeyDown(int KEYID) {
        return keys[KEYID];
    }

    @Override
    public boolean IsKeyJustPressed(int KEYID) {
        boolean b=keysjd[KEYID];
        keysjd[KEYID]=false;
        return b;
    }

    @Override
    public boolean GetMouseDown() {
        return MouseDown;
    }

    @Override
    public boolean GetMouseJustDown() {
        boolean b=MouseJustDown;
        MouseJustDown=false;
        return b;
    }

    @Override
    public void ClearKeys() {
        for (int p=0;p<keysjd.length;p++)
        {
            keysjd[p]=false;
            keys[p]=false;
        }
        MouseJustDown=false;
        typed="";
    }

    @Override
    public void SetTypeBuffer(String s) {
        typed=s;
    }

    @Override
    public String GetTypeBuffer() {
        return typed;
    }

    @Override
    public void Shutdown() {
        Frame.setVisible(false);
    }
    public static class Image_AWT implements IImage {
        protected BufferedImage img,bckimg;

        @Override
        public int GetWidth() {
            return img.getWidth();
        }

        @Override
        public int GetHeight() {
            return img.getHeight();
        }
    }
    @Override
    public IImage GetImage(String a,int tr,int tg,int tb) {
        String ki=a+"_"+tr+"_"+tg+"_"+tb;
        if (ht.containsKey(ki))
        {
            return ht.get(ki);
        }
        try {
            Image_AWT img=new Image_AWT();
            try {
            img.img=ImageIO.read(new File(a));
            } catch (Exception e) {
            img.img=ImageIO.read(ClassLoader.getSystemClassLoader().getResource(a));
            }	
            img.bckimg=new BufferedImage(img.img.getWidth(),img.img.getHeight(),BufferedImage.TYPE_INT_ARGB);
            for (int px=0;px<img.img.getWidth();px++)
            {
            for (int py=0;py<img.img.getHeight();py++)
            {
                int c=img.img.getRGB(px, py);
                if ((c&0xFFFFFF)!=(tb|(tg<<8)|(tr<<16)))
                {
                img.bckimg.setRGB(px, py, c|0xFF000000);
                } else {
                img.bckimg.setRGB(px, py, 0);
                }
            }
            }
            ht.put(ki, img);
            return img;
        } catch (Exception ex) {
    		System.err.println("COULDN'T GET:"+a);
    		ex.printStackTrace();

            Image_AWT img=new Image_AWT();
            
            img.img=new BufferedImage(4,4,BufferedImage.TYPE_INT_RGB);
            img.bckimg=img.img;
            img.img.setRGB(0, 0, 0xFF00FF);
            img.img.setRGB(1, 0, 0xFF00FF);
            img.img.setRGB(0, 1, 0xFF00FF);
            img.img.setRGB(1, 1, 0xFF00FF);
            img.img.setRGB(0+2, 0+2, 0xFF00FF);
            img.img.setRGB(1+2, 0+2, 0xFF00FF);
            img.img.setRGB(0+2, 1+2, 0xFF00FF);
            img.img.setRGB(1+2, 1+2, 0xFF00FF);
            return img;
        }
    }

    @Override
    public void BlitImage(int srcx,int srcy,int srcw,int srch,int x, int y, IImage i) {
        g.drawImage(((Image_AWT)i).img, x, y,(x+srcw),(y+srch),srcx,srcy,(srcx+srcw),(srcy+srch), null);
    }
    @Override
    public void BlitBCKImage(int srcx,int srcy,int srcw,int srch,int x, int y, IImage i) {
        g.drawImage(((Image_AWT)i).bckimg, x, y,(x+srcw),(y+srch),srcx,srcy,(srcx+srcw),(srcy+srch), null);
    }
    
}
